@@ -32,7 +32,6 @@ import org.apache.maven.wagon.repository.Repository;
 import org.apache.maven.wagon.resource.Resource;
 import org.codehaus.plexus.util.FileUtils;
 import org.codehaus.plexus.util.IOUtil;
-import org.codehaus.plexus.util.StringOutputStream;
 import org.codehaus.plexus.util.StringUtils;
 import org.mortbay.jetty.Handler;
 import org.mortbay.jetty.HttpConnection;
@@ -53,6 +52,7 @@ import javax.servlet.ServletException;
 import javax.servlet.ServletInputStream;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
+import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
@@ -67,6 +67,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.zip.GZIPOutputStream;
 
 /**
@@ -162,7 +163,7 @@ public abstract class HttpWagonTestCase
         wagon.connect(
             new Repository( "id", getProtocol() + "://localhost:" + server.getConnectors()[0].getLocalPort() ) );
 
-        wagon.getToStream( "resource", new StringOutputStream() );
+        wagon.getToStream( "resource", new ByteArrayOutputStream() );
 
         wagon.disconnect();
 
@@ -194,7 +195,7 @@ public abstract class HttpWagonTestCase
         wagon.connect(
             new Repository( "id", getProtocol() + "://localhost:" + server.getConnectors()[0].getLocalPort() ) );
 
-        wagon.getToStream( "resource", new StringOutputStream() );
+        wagon.getToStream( "resource", new ByteArrayOutputStream() );
 
         wagon.disconnect();
 
@@ -243,6 +244,73 @@ public abstract class HttpWagonTestCase
         }
     }
 
+    public void testList429()
+        throws Exception
+    {
+        StreamingWagon wagon = (StreamingWagon) getWagon();
+        try
+        {
+
+            Server server = new Server( 0 );
+            final AtomicBoolean called = new AtomicBoolean();
+
+            AbstractHandler handler = new AbstractHandler()
+            {
+                public void handle( String s, HttpServletRequest request, HttpServletResponse response, int i )
+                    throws IOException, ServletException
+                {
+                    if ( called.get() )
+                    {
+                        response.setStatus( 200 );
+                        ( (Request) request ).setHandled( true );
+                    }
+                    else
+                    {
+                        called.set( true );
+                        response.setStatus( 429 );
+                        ( (Request) request ).setHandled( true );
+
+                    }
+                }
+            };
+
+            server.setHandler( handler );
+            addConnectors( server );
+            server.start();
+
+            wagon.connect( new Repository( "id", getRepositoryUrl( server ) ) );
+
+            try
+            {
+                wagon.getFileList( "resource" );
+            }
+            finally
+            {
+                wagon.disconnect();
+
+                server.stop();
+            }
+
+        }
+        catch ( ResourceDoesNotExistException e )
+        {
+            assertTrue( true );
+        }
+        catch ( TransferFailedException e )
+        {
+            if ( wagon.getClass().getName().contains( "Lightweight" ) )
+            {
+                //we don't care about lightweight
+                assertTrue( true );
+            }
+            else
+            {
+                fail();
+            }
+
+        }
+    }
+
     public void testGet500()
         throws Exception
     {
@@ -273,7 +341,7 @@ public abstract class HttpWagonTestCase
 
         try
         {
-            wagon.getToStream( "resource", new StringOutputStream() );
+            wagon.getToStream( "resource", new ByteArrayOutputStream() );
             fail();
         }
         finally
@@ -325,6 +393,60 @@ public abstract class HttpWagonTestCase
         }
     }
 
+    public void testResourceExists429()
+        throws Exception
+    {
+        try
+        {
+
+            final AtomicBoolean called = new AtomicBoolean();
+
+            AbstractHandler handler = new AbstractHandler()
+            {
+                public void handle( String s, HttpServletRequest request, HttpServletResponse response, int i )
+                    throws IOException, ServletException
+                {
+                    if ( called.get() )
+                    {
+                        response.setStatus( HttpServletResponse.SC_INTERNAL_SERVER_ERROR );
+                        ( (Request) request ).setHandled( true );
+                    }
+                    else
+                    {
+                        called.set( true );
+                        response.setStatus( 429 );
+                        ( (Request) request ).setHandled( true );
+                    }
+                }
+            };
+
+            StreamingWagon wagon = (StreamingWagon) getWagon();
+            Server server = new Server( 0 );
+            server.setHandler( handler );
+            addConnectors( server );
+            server.start();
+            wagon.connect( new Repository( "id", getRepositoryUrl( server ) ) );
+
+            try
+            {
+                wagon.resourceExists( "resource" );
+            }
+            finally
+            {
+                wagon.disconnect();
+
+                server.stop();
+            }
+
+            fail();
+        }
+        catch ( TransferFailedException e )
+        {
+            assertTrue( true );
+        }
+    }
+
+
     private boolean runTestResourceExists( int status )
         throws Exception
     {
@@ -958,10 +1080,9 @@ public abstract class HttpWagonTestCase
 
             wagon.connect( testRepository, proxyInfo );
 
-            StringOutputStream out = new StringOutputStream();
             try
             {
-                wagon.getToStream( "test-proxied-resource", out );
+                wagon.getToStream( "test-proxied-resource", new ByteArrayOutputStream() );
 
                 assertTrue( handler.headers.containsKey( "Proxy-Connection" ) );
             }
@@ -1101,7 +1222,7 @@ public abstract class HttpWagonTestCase
 
             wagon.connect( testRepository, authInfo );
 
-            StringOutputStream out = new StringOutputStream();
+            ByteArrayOutputStream out = new ByteArrayOutputStream();
             try
             {
                 wagon.getToStream( "test-secured-resource", out );
@@ -1111,7 +1232,7 @@ public abstract class HttpWagonTestCase
                 wagon.disconnect();
             }
 
-            assertEquals( "top secret", out.toString() );
+            assertEquals( "top secret", out.toString( "US-ASCII" ) );
 
             TestSecurityHandler securityHandler = (TestSecurityHandler) ( (Context) server.getHandler() ).getHandler();
             testPreemptiveAuthenticationGet( securityHandler, supportPreemptiveAuthenticationGet() );
@@ -1294,6 +1415,68 @@ public abstract class HttpWagonTestCase
         }
     }
 
+    public void testPut429()
+        throws Exception
+    {
+
+        try
+        {
+
+            StreamingWagon wagon = (StreamingWagon) getWagon();
+            Server server = new Server( 0 );
+            final AtomicBoolean called = new AtomicBoolean();
+
+            AbstractHandler handler = new AbstractHandler()
+            {
+                public void handle( String s, HttpServletRequest request, HttpServletResponse response, int i )
+                    throws IOException, ServletException
+                {
+                    if ( called.get() )
+                    {
+                        response.setStatus( HttpServletResponse.SC_INTERNAL_SERVER_ERROR );
+                        ( (Request) request ).setHandled( true );
+                    }
+                    else
+                    {
+                        called.set( true );
+                        response.setStatus( 429 );
+                        ( (Request) request ).setHandled( true );
+                    }
+                }
+            };
+
+            server.setHandler( handler );
+            addConnectors( server );
+            server.start();
+
+            wagon.connect( new Repository( "id", getRepositoryUrl( server ) ) );
+
+            File tempFile = File.createTempFile( "wagon", "tmp" );
+            tempFile.deleteOnExit();
+            FileUtils.fileWrite( tempFile.getAbsolutePath(), "content" );
+
+            try
+            {
+                wagon.put( tempFile, "resource" );
+                fail();
+            }
+            finally
+            {
+                wagon.disconnect();
+
+                server.stop();
+
+                tempFile.delete();
+            }
+
+        }
+        catch ( TransferFailedException e )
+        {
+            assertTrue( true );
+        }
+    }
+
+
     private void runTestPut( int status )
         throws Exception
     {
@@ -1706,9 +1889,9 @@ public abstract class HttpWagonTestCase
             throws IOException, ServletException
         {
             headers = new HashMap<String, String>();
-            for ( Enumeration e = request.getHeaderNames(); e.hasMoreElements(); )
+            for ( Enumeration<String> e = request.getHeaderNames(); e.hasMoreElements(); )
             {
-                String name = (String) e.nextElement();
+                String name = e.nextElement();
                 headers.put( name, request.getHeader( name ) );
             }
 
